-- main boss spawner table
BossSpawner = {}

-- boss spawner default configuration
require 'boss_spawner.boss_spawner_config'

--- this will broadcast to all players within BroadcastMessageRange
-- @param bossSpawnerKey - this is the Boss Key in the bossspawner table
-- @param stage - the current boss spawn stage
function BroadCastMessage(bossSpawnerKey,stage)
	if(BossSpawner[bossSpawnerKey].Stages[stage].StartWaveMessage == nil) then return end
	local mobiles = FindObjects(SearchMobileInRange(BossSpawner.Config.BroadcastMessageRange))
	for k,v in pairs(mobiles) do
		if(v:IsPlayer()) then v:SystemMessage(BossSpawner[bossSpawnerKey].Stages[stage].StartWaveMessage,"event") end
	end
end

--- return the timer pulse frequency
function GetSpawnPulse()
	if(this:GetObjVar("Data.Sleeping") == true) then
		return TimeSpan.FromSeconds(BossSpawner.Config.TimerPulseWhileSleeping + math.random())
	else
		return TimeSpan.FromSeconds(BossSpawner.Config.TimerPulseWhileAwake + math.random())
	end
end

--- fires when a object is spawned
-- @param success - if the object is correctly created and valid
-- @param objref - the object reference that is spawned
function HandleCreatedObject(success, objref)
	if(success and objref) then
		local isMobile = objref:IsMobile()
		local spawnTable = this:GetObjVar("Data.SpawnTable")
		table.insert(spawnTable,objref)
		this:SetObjVar("Data.SpawnTable",spawnTable)
		PlayEffectAtLoc(BossSpawner.Config.VisualEffect.Name,objref:GetLoc(), BossSpawner.Config.VisualEffect.Duration)
		if(isMobile) then
			if(BossSpawner[this:GetObjVar("Data.BossKey")].MobileScaleTimes) then
				local scale = BossSpawner[this:GetObjVar("Data.BossKey")].MobileScaleTimes or 1
				objref:SetScale(Loc(scale,scale,scale))
			end	
			--NOTE: should we append loot tables as an option instead? 	
			if(BossSpawner[this:GetObjVar("Data.BossKey")].RemoveLootPacks == true) then
	        	CallFunctionDelayed(TimeSpan.FromSeconds(0.3),function() 
	        		objref:DelObjVar("LootTables") 
	        		if(BossSpawner[this:GetObjVar("Data.BossKey")].AddLootPacks) then
	        			local lootPackTable = {}
	        			for lootTableIndex,lootTableKey in pairs(BossSpawner[this:GetObjVar("Data.BossKey")].AddLootPacks) do
	        				-- does the lootpack exist? if so add it to the loot table
	        				if(TemplateDefines.LootTable[lootTableKey]) then
	        					table.insert(lootPackTable,TemplateDefines.LootTable[lootTableKey])
	        				end
	        			end
	        			if(#lootPackTable > 0) then
	        				objref:SetObjVar("LootTables",lootPackTable)
	        			end
	        		end
	        	end)
			end
			if(BossSpawner[this:GetObjVar("Data.BossKey")].MobileTeamType) then
				objref:SetObjVar("MobileTeamType",BossSpawner[this:GetObjVar("Data.BossKey")].MobileTeamType)
			else
				objref:SetObjVar("MobileTeamType","BossSpawner")
			end
			if(BossSpawner[this:GetObjVar("Data.BossKey")].MobileTitle) then
				objref:SetSharedObjectProperty("Title",BossSpawner[this:GetObjVar("Data.BossKey")].MobileTitle)
			end
			if(BossSpawner[this:GetObjVar("Data.BossKey")].CantBeTamed) then
				objref:DelObjVar("TamingDifficulty")
			end
		end
	end
end

--- fires when this module is attached to the boss spawner object
function HandleModuleAttached()
	if not(this:HasObjVar("Data.BossKey")) then
		this:SetObjVar("Data.BossKey","SimpleUndeadSpawn")
	end
	this:SetObjVar("Data.SpawnTable",{})
	this:SetObjVar("Data.DelayInSeconds", (math.random(BossSpawner.Config.TimerRestartDelay.Min,BossSpawner.Config.TimerRestartDelay.Max)) or 3600)
	this:SetObjVar("Data.CurrentWave",0)
	this:SetObjVar("Data.CurrentStage",BossSpawner[this:GetObjVar("Data.BossKey")].StartingStage)
	this:SetObjVar("Data.RestartTime", os.time() + this:GetObjVar("Data.DelayInSeconds"))
	this:SetObjVar("Data.Sleeping",false)
end

--- check the spawn conditions
function HandleSpawnTimer()
	if(this:GetObjVar("Data.Sleeping") == true) then
		local restartTime = this:GetObjVar("Data.RestartTime")
		if(restartTime < os.time()) then ResetBossSpawner() end	 
		this:ScheduleTimerDelay(GetSpawnPulse(), "checkSpawner")
		UpdateBossSpawnerTooltip()
		return 
	end
	if(IsSpawnDead()) then
		local currentWave = this:GetObjVar("Data.CurrentWave") or 1
		local key = this:GetObjVar("Data.BossKey") or "Mogus"
		local stageKey = this:GetObjVar("Data.CurrentStage") or BossSpawner[key].StartingStage
		--- if starting up for first run, broadcast the start wave message
		if(currentWave == 0 and stageKey == BossSpawner[key].StartingStage) then
			BroadCastMessage(this:GetObjVar("Data.BossKey"),BossSpawner[this:GetObjVar("Data.BossKey")].StartingStage)
		end
		--- updage wave counter and save it
		currentWave = currentWave + 1
		this:SetObjVar("Data.CurrentWave",currentWave)
		--- if we done all stages, update stage info
		if(currentWave > BossSpawner[key].Stages[stageKey].TotalWaves) then
			--- reset the wave counter
			this:SetObjVar("Data.CurrentWave",1)
			--- set the new current stage
			this:SetObjVar("Data.CurrentStage",BossSpawner[key].Stages[stageKey].NextStage)
			stageKey = BossSpawner[key].Stages[stageKey].NextStage
			--only broadcast if there is a next wave
			if(BossSpawner[key].Stages[stageKey]) then 
				BroadCastMessage(key,stageKey) 
			end
		end
		--- if there is a stageKey (NextStage) then spawn the wave
		--- else this must be the end of the wave. check reset time.
		if (stageKey ~= nil and stageKey ~= "EndStage") then 
			SpawnWave(key,stageKey) 
		end
		--- lastly if its the end stage, lets kill everything and put the spawner to sleep
		if (stageKey == "EndStage") then
			if(BossSpawner[this:GetObjVar("Data.BossKey")].OnCompletionDestroySelf == true) then		
				this:Destroy()
				return
			end		 
			if(tostring(this:GetIconId()) == "2") then this:SetColor("752525") end
			this:SetObjVar("Data.Sleeping",true) 
			this:SetObjVar("Data.RestartTime", os.time() + this:GetObjVar("Data.DelayInSeconds"))
		end
	end
	--- start the timer up
	this:ScheduleTimerDelay(GetSpawnPulse(), "checkSpawner")
	UpdateBossSpawnerTooltip()
end

--- checks to see if all the mobs are dead
function IsSpawnDead()
	local spawnTable = this:GetObjVar("Data.SpawnTable")
	if(spawnTable) then
		local tableCount = #this:GetObjVar("Data.SpawnTable")
		local deadCount = 0
		for k,v in pairs(spawnTable) do
			if((v:IsMobile() and IsDead(v)) or not v:IsValid()) then deadCount = deadCount + 1 end
		end
		return (deadCount == tableCount)
	else
		return false
	end
end

--- reset the boss spawner for the next wave
function ResetBossSpawner()
	local newDelay = math.random(BossSpawner.Config.TimerRestartDelay.Min,BossSpawner.Config.TimerRestartDelay.Max) or 3600
	this:SetObjVar("Data.SpawnTable",{})
	this:SetObjVar("Data.CurrentWave",0)
	this:SetObjVar("Data.DelayInSeconds", newDelay)
	this:SetObjVar("Data.RestartTime",os.time() + newDelay)
	this:SetObjVar("Data.Sleeping",false)
	this:SetObjVar("Data.CurrentStage",BossSpawner[this:GetObjVar("Data.BossKey")].StartingStage)
	if(tostring(this:GetIconId()) == "2") then this:SetColor("257525") end
	BroadCastMessage(this:GetObjVar("Data.BossKey"),BossSpawner[this:GetObjVar("Data.BossKey")].StartingStage)
end

--- this will broadcast to all players within BroadcastMessageRange
-- @param bossSpawnerKey - this is the Boss Key in the bossspawner table
-- @param stage - the current boss spawn stage
function SpawnWave(bossSpawnerKey,stage)
	--calculate spawn amount, make sure at least 1 spawns
	local spawnAmount = BossSpawner[bossSpawnerKey].Stages[stage].MonstersPerWave.Base + BossSpawner[bossSpawnerKey].Stages[stage].MonstersPerWave.Bonus * this:GetObjVar("Data.CurrentWave") or 1
	for i=1, spawnAmount do
		local template = BossSpawner[bossSpawnerKey].Stages[stage].MonsterList[math.random(#BossSpawner[bossSpawnerKey].Stages[stage].MonsterList)]
		local range = BossSpawner.Config.SpawnRange
		if(BossSpawner[bossSpawnerKey].Stages[stage].Range ~= nil) then
			range = math.random(BossSpawner[bossSpawnerKey].Stages[stage].Range.Min,BossSpawner[bossSpawnerKey].Stages[stage].Range.Max)
		end
		local location = GetRandomPassableLocationInRadius(this:GetLoc(),range,true)	
		CreateObj(template, location, "mobSpawned")
	end
end

function UpdateBossSpawnerTooltip()
	if(tostring(this:GetIconId()) ~= "2") then 
		if(BossSpawner[this:GetObjVar("Data.BossKey")].CustomToolTip) then
			RemoveTooltipEntry(this,"bossSpawnerCustomToolTip")
			if(this:GetObjVar("Data.Sleeping") == false) then	
	    		SetTooltipEntry(this,"bossSpawnerCustomToolTip",BossSpawner[this:GetObjVar("Data.BossKey")].CustomToolTip.Active,1100)
			else
	    		SetTooltipEntry(this,"bossSpawnerCustomToolTip",BossSpawner[this:GetObjVar("Data.BossKey")].CustomToolTip.Sleeping,1100)
			end
		end
		return 
	end
    local descriptionColor = "[959555]"    
    local valueColor = "[c5c595]"
    local IsSleeping = this:GetObjVar("Data.Sleeping")
    RemoveTooltipEntry(this,"bossSpawnerTopText")
    SetTooltipEntry(this,"bossSpawnerTopText","[FF7700] Boss Spawner Details",1100)
    if(IsSleeping == false) then
        RemoveTooltipEntry(this,"bossSpawnerSleepState")
        SetTooltipEntry(this,"bossSpawnerSleepState","[55AA55]*Awake*",1000)
    else
        RemoveTooltipEntry(this,"bossSpawnerSleepState")
        SetTooltipEntry(this,"bossSpawnerSleepState","[AA5555]*Sleeping*",1000)
    end
    RemoveTooltipEntry(this,"bossSpawnerBossKey")             
    SetTooltipEntry(this,"bossSpawnerBossKey",descriptionColor.."Table Key: ".. valueColor .. this:GetObjVar("Data.BossKey"),990)
    RemoveTooltipEntry(this,"bossSpawnerCurrentStage")             
    SetTooltipEntry(this,"bossSpawnerCurrentStage",descriptionColor.."Current Stage: ".. valueColor .. this:GetObjVar("Data.CurrentStage"),980)
    RemoveTooltipEntry(this,"bossSpawnerCurrentWave")             
    SetTooltipEntry(this,"bossSpawnerCurrentWave",descriptionColor.."Current Wave: ".. valueColor .. this:GetObjVar("Data.CurrentWave"),970)
    RemoveTooltipEntry(this,"bossSpawnerNextRestartDate")             
    SetTooltipEntry(this,"bossSpawnerNextRestartDate",descriptionColor.."Next Restart Date: ".. valueColor .. os.date("%x", this:GetObjVar("Data.RestartTime")),960)
    RemoveTooltipEntry(this,"bossSpawnerNextRestartTime")             
    SetTooltipEntry(this,"bossSpawnerNextRestartTime",descriptionColor.."Next Restart Time: ".. valueColor .. os.date("%X %p", this:GetObjVar("Data.RestartTime")),950)
end

RegisterEventHandler(EventType.CreatedObject, "mobSpawned", function(...) HandleCreatedObject(...) end)
RegisterEventHandler(EventType.ModuleAttached, GetCurrentModule(), function(...) HandleModuleAttached(...) end)
RegisterEventHandler(EventType.Timer, "checkSpawner", function(...) HandleSpawnTimer(...) end)

-- as soon as script attaches and fires, kick off the timer
this:ScheduleTimerDelay(GetSpawnPulse(), "checkSpawner")

