--- When crafting something out of a material (vs a resource) the resource get a weighted roll dependant on the type of material
--- Note: This is expected to be called BEFORE setting tooltips on the item!
-- @param objRef(GameObj) The recently created object.
-- @param material(string) The string name of the material the item was created from
-- @param skillLevel(number) The skill level of the skill used to create the item
-- @return none
function ApplyCraftedMaterialProperties(objRef, material, skillLevel)
	if ( objRef == nil ) then
		LuaDebugCallStack("[ApplyCraftedMaterialProperties] Nil objRef provided.")
		return
	end
	if ( Materials[material] == nil ) then
		LuaDebugCallStack("[ApplyCraftedMaterialProperties] Invalid material '"..tostring(material).."' provided.")
		return
	end
	if ( skillLevel == nil ) then
		LuaDebugCallStack("[ApplyCraftedMaterialProperties] Invalid skillLevel '"..tostring(skillLevel).."' provided.")
		return
	end
	local skillPercent = skillLevel / ServerSettings.Skills.PlayerSkillCap.Single

	-- pre-pend the name with the material
	local name = StripColorFromString(objRef:GetName())
	if ( ResourceData.ResourceInfo[material].CraftedItemPrefix ~= nil ) then
		objRef:SetName(MaterialTooltipColors[material]..ResourceData.ResourceInfo[material].CraftedItemPrefix.." "..name.."[-]")
	else
		objRef:SetName(name)
	end
	-- set the color from material
	objRef:SetHue(Materials[material])
	-- set an objvar of the type of material
	objRef:SetObjVar("Material", material)
	-- increase the max durability
	if ( MaterialProperties.Config.BonusDurability[material] ) then
		local durability = GetMaxDurabilityValue(objRef)
		objRef:SetObjVar("MaxDurability", durability + math.random(MaterialProperties.Config.BonusDurability[material].Min,MaterialProperties.Config.BonusDurability[material].Max) + math.random(1,10))
	end

	-- add weapon bonuses
	if ( objRef:HasObjVar("WeaponType") and MaterialProperties.Config.BonusWeaponAttackDamage[material]) then
		objRef:SetObjVar("AttackBonus", math.random(MaterialProperties.Config.BonusWeaponAttackDamage[material].Min,MaterialProperties.Config.BonusWeaponAttackDamage[material].Max))
	end

	-- add armor bonuses
	if ( objRef:HasObjVar("ArmorType") or objRef:HasObjVar("ShieldType")  and MaterialProperties.Config.BonusArmorDefense[material]) then
		objRef:SetObjVar("ArmorBonus", math.random(MaterialProperties.Config.BonusArmorDefense[material].Min,MaterialProperties.Config.BonusArmorDefense[material].Max))
	end

	-- add material bonus if its not a basic tier 1 material
	if(objRef:HasObjVar("ArmorType") or objRef:HasObjVar("ShieldType") or objRef:HasObjVar("WeaponType")) then
		-- does the material table exist?
		if(MaterialProperties[material]) then
			ApplyMaterialProperties(objRef,material,skillLevel)
			-- set the tooltip again on the item
			SetItemTooltip(objRef)
		end
	end
end

