-- this file is loaded up from the scripts/globals/helpers/crafting.lua
---------------------------------------------------------------------------------------
-- MaterialProperties Table
---------------------------------------------------------------------------------------
-- this table contains all the Material Bonus modifications information
MaterialProperties = {}

---------------------------------------------------------------------------------------
-- helpers.crafting.lua overrides
---------------------------------------------------------------------------------------
require 'material_properties.crafting_overrides'

---------------------------------------------------------------------------------------
-- Material Property Configuration
---------------------------------------------------------------------------------------
require 'material_properties.material_config'

---------------------------------------------------------------------------------------
-- Material Definitions
---------------------------------------------------------------------------------------
require 'material_properties.definitions.ash_material'
require 'material_properties.definitions.beast_material'
require 'material_properties.definitions.blightwood_material'
require 'material_properties.definitions.cobalt_material'
require 'material_properties.definitions.copper_material'
require 'material_properties.definitions.gold_material'
require 'material_properties.definitions.obsidian_material'
require 'material_properties.definitions.quilted_material'
require 'material_properties.definitions.silk_material'
require 'material_properties.definitions.vile_material'

---------------------------------------------------------------------------------------
-- Material Crafting Generation
---------------------------------------------------------------------------------------
--- apply material properites to the crafted item
-- @param equipmentObject - (GameObject) the object that was just crafted
-- @param material - (String) the material being crafted
-- @param skillLevel - (Number) the skill level of crafter of the object
function ApplyMaterialProperties(equipmentObject,material,skillLevel)
	-- can we apply the material bonuses?
	if not(((MaterialProperties.Config.ChanceToApply[material] or 0) >= math.random(1,100))) then return end
	
	if(equipmentObject:HasObjVar("Enchants")) then return end

	-- build enchants table for the item
	local enchants = {}
	
	-- get the randomly generated property count of the item
	local propertyCount = math.random( (MaterialProperties.Config.PropertyCount[material].Min or 1), (MaterialProperties.Config.PropertyCount[material].Max or 1) )
	
	local itemType = ""
	if(equipmentObject:HasObjVar("WeaponType")) then itemType = "WeaponType" end
	if(equipmentObject:HasObjVar("WeaponType")) then itemType = "WeaponType" end
	if(equipmentObject:HasObjVar("ArmorType")) then itemType = "ArmorType" end

	-- crazy using while???? but its okay due to how generation works.
	while(#enchants < propertyCount) do
		-- generate a fresh random table to use
		local essenceTable = {}

		-- generate the essenceName
		essenceTable.essenceName = MaterialProperties[material][itemType][math.random(1,#MaterialProperties[material][itemType])]
		
		-- generate the value for our property
		if(EssenceData[essenceTable.essenceName].EffectRangeMax < 1) then
			local maxRange = EssenceData[essenceTable.essenceName].EffectRangeMax * 100000
			local rndNum = math.random(maxRange)/100000
			essenceTable.essenceModifier = rndNum
		else
			essenceTable.essenceModifier = math.random(1,EssenceData[essenceTable.essenceName].EffectRangeMax)	
		end

		-- if this is the first enchant, then add it other wise check to see if we have it
		if(#enchants < 1) then
			--first enchant so place it
			table.insert(enchants,essenceTable)
		else
			-- this variable allows us to have the generator try again if it does not meet
			-- the requirements below
			local canApplyEnchant = true
			for enchantIndex,enchantTable in ipairs(enchants) do
				-- do we have this enchant? if so dont add it
				if(essenceTable.essenceName == enchantTable.essenceName) then
					canApplyEnchant = false
				end						
			end
			
			-- if we can apply it then do so
			if(canApplyEnchant==true) then
				table.insert(enchants,essenceTable)
			end
		end
	end
	-- apply the enchants to the item
	if(#enchants>0) then equipmentObject:SetObjVar("Enchants",enchants) end
end
